#include "neuroninterz.h"

NeuronInterz::NeuronInterz()
{

// 	maxSynapses			= 150;

	isInhibitory			= false;

	// input weight range, 5 = -5 <-> +5	-> 10 in total because 0 will be excluded
// 	dendridicBranches		= 10;

	// processing
// 	firingThreshold			= 30.0f;
	potential			= 0.0f;
	potentialDecay			= 0.95f;

	// output
	output				= 0;
	waitoutput			= 0;

	// plasticity up & down
	isPlastic			= false;
// 	plasticityStrengthen		= 1.0f+(1.0f/100.0f);
// 	plasticityWeaken		= 1.0f-(1.0f/1000.0f);

	// SPECIALO optional reference that makes this a MOTOR neuron, but it depends on not being defined
	isMotor				= false;
	motorFunc			= 0;

        settings = Settings::Instance();	
	pill_maxtrip = settings->getCVarPtr("pill_maxtrip");
}

void NeuronInterz::process(float energy, float* tripping, Circuit8* circuit8, unsigned int RuRdGuGdBuBd, unsigned int screenOffsetX, unsigned int screenOffsetY)
//void NeuronInterz::process(float* tripping, Circuit8* circuit8, unsigned int RuRdGuGdBuBd, unsigned int screenOffsetX, unsigned int screenOffsetY)
//float NeuronInterz::process(float* tripping, Circuit8* circuit8, unsigned int RuRdGuGdBuBd, unsigned int screenOffsetX, unsigned int screenOffsetY)
{
//	if (tripping != 0) {
//		printf("tripping value is %f\n",tripping);
//		}
	// potential decay
	potential = potential * potentialDecay;

//	cout << "neuroni energy "<< energy << endl;

//	cout << "neuroni " << circuit8->screenidx << endl;

	// morphogenic field (first area of screen) effect per neuron
	if (RuRdGuGdBuBd == 0) { potential += circuit8->screen.screen[screenOffsetX][screenOffsetY].r[circuit8->screenidx] / 100; }
	else if (RuRdGuGdBuBd == 1) { potential -= circuit8->screen.screen[screenOffsetX][screenOffsetY].r[circuit8->screenidx] / 100; }
	else if (RuRdGuGdBuBd == 2) { potential += circuit8->screen.screen[screenOffsetX][screenOffsetY].g[circuit8->screenidx] / 100; }
	else if (RuRdGuGdBuBd == 3) { potential -= circuit8->screen.screen[screenOffsetX][screenOffsetY].g[circuit8->screenidx] / 100; }
	else if (RuRdGuGdBuBd == 4) { potential += circuit8->screen.screen[screenOffsetX][screenOffsetY].b[circuit8->screenidx] / 100; }
	else if (RuRdGuGdBuBd == 5) { potential -= circuit8->screen.screen[screenOffsetX][screenOffsetY].b[circuit8->screenidx] / 100; }

	RuRdGuGdBuBd += 1;

	// make every connection do it's influence on the neuron's total potential
	unsigned int ssize = Synapses.size();
	for ( unsigned int i=0; i < ssize; i++ )
	{
//		cout << "neuron influenced by synapse " << i << " out of " << ssize << endl;
		// lower synaptic weights
		if (isPlastic) {
//			cout << "synapse was plastic, " << endl;
//			cout << "weight is " << Synapses[i].weight << endl;
			Synapses[i].weight = Synapses[i].weight * plasticityWeaken;
		}

//		cout << "crash here probably means the synapse is connected to an inexistent neuron id" << endl;
		
		if ( *Synapses[i].ref != 0 ) {
//			cout << "Synapses[i].ref != 0" << endl;
			potential += (Synapses[i].weight * Synapses[i].dendriteBranch * *Synapses[i].ref);
		}
	}

//	cout << "checking inhibitory/excitatory" << endl;

	float alien_ai_fleet_seed;
	if ( isInhibitory )
	{
//		cout << "neuron is inhib" << endl;

		if (tripping[0]) { // inhibitory antagonism
			alien_ai_fleet_seed = tripping[0];
			potential = potential * (1 + (tripping[0] / (10000.0f - ((tripping[4] / *pill_maxtrip)*10000.0f))));
			tripping[0] = alien_ai_fleet_seed;
		}

		if (tripping[1]) { // inhibitory agonism
			alien_ai_fleet_seed = tripping[1];
//			potential = potential - (tripping[1] / (1000.0f - ((tripping[4] / *pill_maxtrip)*1000.0f)));
			potential = potential * (1 - (tripping[1] / (10000.0f - ((tripping[4] / *pill_maxtrip)*10000.0f))));
			tripping[1] = alien_ai_fleet_seed;                // whats happening to value of tripping that i need to do this (flamoot)
		}

		// give them some potential if they're well fed
//		cout << "would have mionused "<<energy<<" to "<<potential<< " potential" << endl;
		potential -= energy;

//		printf("tripping %f %f\n",tripping,potential);
//		cout << "I" << potential << "\n";

		// do we spike/fire
		if ( potential <= -1.0f * firingThreshold )
		{
			// reset neural potential
			potential = 0.0f;
	
			// fire the neuron
			waitoutput = -1;
			
//			cout << "neuron fires" << endl;

			// PLASTICITY: if neuron & synapse fire together, the synapse strenghtens
			if (isPlastic)
			{
				for ( unsigned int i=0; i < ssize; i++ )
				{
//					cout << "synapse " << i << " of " << ssize << " is plastic inhib" << endl;
					// if synapse fired, strenghten the weight
					if ( (*Synapses[i].ref < 0.0f && Synapses[i].weight > 0.0f) || (*Synapses[i].ref > 0.0f && Synapses[i].weight < 0.0f) )
					{
// 						cerr << endl << "Inhibitory firing" << endl << "synref: " << *Synapses[i].ref << endl << "pre weight:  " << Synapses[i].weight << endl;
						Synapses[i].weight = Synapses[i].weight * plasticityStrengthen;
// 						cerr << "post weight: " << Synapses[i].weight << endl;
					}

					// if weight > max back to max
 					if	( Synapses[i].weight > 5.0f ) Synapses[i].weight = 5.0f;
 					else if ( Synapses[i].weight <-5.0f ) Synapses[i].weight =-5.0f;
				}
			}
		}
		// don't fire the neuron
		else
		{
			waitoutput = 0;
			// reset potential if < 0
			if ( potential > 0.0f ) potential = 0.0f;
		}
	}
	else
	{
//		cout << "neuron is excitatory" << endl;
		if (tripping[2]) { // excitatory antagonism
			alien_ai_fleet_seed = tripping[2];
			potential = potential * (1 - (tripping[2] / (10000.0f - ((tripping[4] / *pill_maxtrip)*10000.0f))));
			tripping[2] = alien_ai_fleet_seed;
		}		

		if (tripping[3]) { // excitatory agonism
			alien_ai_fleet_seed = tripping[3];
			potential = potential * (1 + (tripping[3] / (10000.0f - ((tripping[4] / *pill_maxtrip)*10000.0f))));
			tripping[3] = alien_ai_fleet_seed;
		}
		
		// give them some potential if they're well fed
//		cout << "would have added "<<energy<<" to "<<potential<< " potential" << endl;
		potential += energy;
		

//		printf("tripping %f %f\n",tripping,potential);
//		cout << "E" << potential << "\n";

		// do we spike/fire
		if ( potential >= firingThreshold )
		{
			// reset neural potential
			potential = 0.0f;
	
			// fire the neuron
			waitoutput = 1;
			
//			cout << "neuron fires" << endl;

			// PLASTICITY: if neuron & synapse fire together, the synapse strenghtens
			if (isPlastic)
			{
				for ( unsigned int i=0; i < ssize; i++ )
				{
//					cout << "synapse " << i << " of " << ssize << " is excit plastic" << endl;
					// if synapse fired, strenghten the weight
					if ( (*Synapses[i].ref > 0.0f && Synapses[i].weight > 0.0f) || (*Synapses[i].ref < 0.0f && Synapses[i].weight < 0.0f) )
					{
// 						cerr << endl << "Excititory firing" << endl << "synref: " << *Synapses[i].ref << endl << "pre weight:  " << Synapses[i].weight << endl;
						Synapses[i].weight = Synapses[i].weight * plasticityStrengthen;
// 						cerr << "post weight: " << Synapses[i].weight << endl;
					}

					// if weight > max back to max
 					if	( Synapses[i].weight > 5.0f ) Synapses[i].weight = 5.0f;
 					else if ( Synapses[i].weight <-5.0f ) Synapses[i].weight =-5.0f;
				}
			}
		}
		// don't fire the neuron
		else
		{
			waitoutput = 0;
			// reset potential if < 0
			if ( potential < 0.0f ) potential = 0.0f;
		}
		//if (tripping) { tripping -= 0.00001f; }
	}
	// neuron effect on morphogenic field (first area of screen)
	
//	cout << "finished synapses" << endl;
	
	if (RuRdGuGdBuBd == 1) { circuit8->lessRed(screenOffsetX, screenOffsetY, potential / 50000); }
	else if (RuRdGuGdBuBd == 2) { circuit8->moreGreen(screenOffsetX, screenOffsetY, potential / 50000); }
	else if (RuRdGuGdBuBd == 3) { circuit8->lessGreen(screenOffsetX, screenOffsetY, potential / 50000); }
	else if (RuRdGuGdBuBd == 4) { circuit8->moreBlue(screenOffsetX, screenOffsetY, potential / 50000); }
	else if (RuRdGuGdBuBd == 5) { circuit8->lessBlue(screenOffsetX, screenOffsetY, potential / 50000); }
	else if (RuRdGuGdBuBd == 0 || RuRdGuGdBuBd == 6) { circuit8->moreRed(screenOffsetX, screenOffsetY, potential / 50000); }

//	return tripping; // unchanged anyway and is a vector now so i dont know how anymore
//	return potential; // testing
//	cout << "neuroni energy @ exit "<< energy << endl;

}

void NeuronInterz::connec( float *output, unsigned int dendriteBranch, float synapticWeight )
//void NeuronInterz::connec( int *output, unsigned int dendriteBranch, float synapticWeight )
{
		Synapse s;
		s.ref = output;
		s.dendriteBranch = dendriteBranch;
		s.weight = synapticWeight;

		Synapses.push_back( s );

//		cerr << "I: " << Synapses.size()-1 << "=" << *Synapses[Synapses.size()-1].ref << "(" << Synapses[Synapses.size()-1].ref << ")";
//		cerr << *Synapses[0].ref << endl;
}

NeuronInterz::~NeuronInterz()
{
//	for ( unsigned int i=0; i < inputs.size(); i++ ) delete inputs[i];
}


